use search::apply_move;
use bitboard::Bitboard;
use game::Game;
use search::*;
use movegen::*;
use log::{info};
use std::time::{Duration, Instant};
use std::collections::VecDeque;



use std::sync::mpsc::{Receiver, Sender};

pub enum EngineMsg {
    SetBoard(Game),
    SetPiece(Bitboard),
    Search(i32),
    Ping,
    Stop,
    NewGame,
    GetState,
    GetInfo,
}


pub enum EngineState {
    Idle,
    Searching
}


pub enum InterfaceMsg {
    BestMove(Move),
    State(EngineState)
}


fn dequeue_message(queue: &mut VecDeque<EngineMsg>, r: &Receiver<EngineMsg>) -> Option<EngineMsg> {
    if queue.is_empty() {
        r.recv().ok()
    }
    else {
        queue.pop_front()
    }
}

pub fn run_engine(r: Receiver<EngineMsg>, s: Sender<InterfaceMsg>) {
    let mut game = Game::default();
    let mut messages: VecDeque<EngineMsg> = VecDeque::new();
    loop {
        let msg = dequeue_message(&mut messages, &r).unwrap();

        match msg {
            EngineMsg::SetBoard(g) => {
                //println!("SetBoard");
                game = g;
            },
            EngineMsg::SetPiece(_) => { println!("SetPiece") },
            EngineMsg::Search(depth) => {
                //info!("searching in pos\n {}", game.beautiful_print());
                //println!("Search {}", depth);

                let mut check_fn = || -> bool {
                    if let Ok(msg) = r.try_recv() {
                        if let EngineMsg::Stop = msg {
                            return true;
                        }
                        else {
                            messages.push_back(msg);
                        }
                    }
                    return false;
                };

                let mut sc = SearchControl{ nodes: 0, check: &mut check_fn };

                let before = Instant::now();
                let best_move = search(&game, &mut sc, depth);
                let time = before.elapsed();
                println!("bestmove {}", best_move.to_string());
                info!("bestmove {}", best_move.to_string());
                info!("searched {} nodes in {} ms ({} nodes/s)", sc.nodes, time.as_millis(), (sc.nodes as f64 / time.as_millis() as f64 * 1000.0) as i64);
            },
            EngineMsg::Ping => { println!("Ping") },
            EngineMsg::Stop => { println!("Stop") },
            EngineMsg::NewGame => {
                //println!("NewGame")
            },
            EngineMsg::GetState => { println!("GetState") },
            EngineMsg::GetInfo => { println!("GetInfo") },
        }

        //println!("{}", game.beautiful_print());

        //let moves = generate_moves(&game, WHITE);
    }
}

